import json

from flask import render_template, session, abort, request, redirect, url_for, current_app, flash, jsonify, \
    make_response
from flask_login import login_required

from pyfw import db
from pyfw.main.models import CommonUserInfo
from pyfw.util.JsonUtil import JsonStrToObj, CopyObj, GetResult
from . import api_1_1_system


#@api_1_1_system.after_request
def af_request(resp):
    """
    #请求钩子，在所有的请求发生后执行，加入headers。
    :param resp:
    :return:
    """
    resp = make_response(resp)
    resp.headers['Access-Control-Allow-Origin'] = '*'
    resp.headers['Access-Control-Allow-Methods'] = 'GET,POST'
    resp.headers['Access-Control-Allow-Headers'] = 'x-requested-with,content-type'
    return resp


@api_1_1_system.route('/commonuserinfos', methods=['GET'])
def getList():
    """
    获取数据列表
    :return:
    """

    # 查询字符串
    filters = set()

    # 构建查询条件
    user_name = request.args.get('name')

    if user_name: filters.add(CommonUserInfo.login_account.contains(user_name))

    # 获取查询页
    page = request.args.get('currentPage', 1, type=int)

    pagination = getUserList(filters,page)

    return jsonify(GetResult(pagination))




@api_1_1_system.route('/commonuserinfo', methods=['post'])
def post():
    """
    新增对象
    :param self:
    :return:
    """
    result="0000"
    msg=""

    #构建查询条件
    jsonStr=request.data

    user = CommonUserInfo();

    method,user = JsonStrToObj(jsonStr,user)


    if method=="post":

        user.password=user.login_account

        db.session.add(user);

        msg="新增成功"


    elif method == "update":

        target = CommonUserInfo.query.get_or_404(int(user.id))

        target=CopyObj(user,target)

        db.session.add(target);

        msg = "修改成功"

    elif method == "delete":

        target = CommonUserInfo.query.get_or_404(int(user.id))
        db.session.delete(target)

        msg = "删除成功"

    else:
        print("无任何操作")




    return jsonify(GetResult(getUserList(),{
            "code": "0000",
            "msg":msg
     }))




def getUserList(filters=None,page=1):


    if filters==None:
        filters = set()



    pagination = CommonUserInfo.query.filter(*filters).order_by(CommonUserInfo.id.desc()).paginate(
        page, per_page=current_app.config['FLASKY_POSTS_PER_PAGE'],
        error_out=False)

    return pagination



@api_1_1_system.route('/login', methods=['POST'])
def login():
    """
       登录方法
       :return:
       """

    #构建查询条件
    jsonStr=request.data


    ctdict = json.loads(jsonStr)


    if request.method == 'POST':

        userdb = CommonUserInfo.query.filter_by(login_account=ctdict["userName"]).first()

        current_app.logger.info("===登录日志===")

        if userdb is not None and userdb.verify_password(ctdict["password"]):
            return jsonify({"status": "ok", "type": "account", "currentAuthority": "admin"})

        else:
           pass


    return jsonify({"status": "error", "type": "account", "currentAuthority": "guest", "token": "wenanguo123456"})




#
# @api_1_1_system.route('/commonuserinfo/<int:id>', methods=['get'])
# def get(id):
#         """
#         获得单对象
#         :param id:
#         :return:
#         """
#
#
#
#         #查询字符串
#         filters=set()
#
#         #构建查询条件
#         user_name = request.args.get('name')
#
#         if user_name :filters.add(CommonUserInfo.login_account.contains(user_name))
#
#
#         #获取查询页
#         page = request.args.get('page', 1, type=int)
#
#
#
#
#
#         pagination = CommonUserInfo.query.filter(*filters).order_by(CommonUserInfo.id.desc()).paginate(
#             page, per_page=current_app.config['FLASKY_POSTS_PER_PAGE'],
#             error_out=False)
#
#
#
#
#
#         return jsonify({
#             'items': [user.to_json() for user in pagination.items],
#             'page': pagination.page,
#             'prev_num': pagination.prev_num,
#             'next_num': pagination.next_num,
#             'has_next' :pagination.has_next, #如果有下一页，返回True
#             'has_prev': pagination.has_prev, #如果有上一页，返回True
#             'pages': pagination.pages, #查询得到的总页数
#             'per_page': pagination.per_page, #每页显示的记录数量
#             'total': pagination.total,
#             'iter_pages':[pages for pages in pagination.iter_pages(left_edge = 2 ,left_current=2,right_current=3,right_edge=2)]
#
#         })

#
# @api_1_1_system.route('/commonuserinfo/<int:id>', methods=['put'])
# def put( id):
#         """
#         更新对象
#         :param id:
#         :return:
#         """
#
#         login_account = request.form['login_account']
#
#
#         return jsonify({
#             'result': id,
#             'login_account':login_account,
#             'msg': '修改成功'
#         })
#
#
#
# @api_1_1_system.route('/commonuserinfo/<int:id>', methods=['delete'])
# def delete( id):
#         """
#         删除对象
#         :param id:
#         :return:
#         """
#
#
#
#
#         return jsonify({
#             'result': id,
#             'msg': '删除成功'
#         })


#!/usr/bin/env python
# -*- coding: utf-8 -*-

' json序列化工具对象'
import datetime
import json



class CJsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, datetime.date):
            return obj.strftime("%Y-%m-%d")
        else:
            return json.JSONEncoder.default(self, obj)

__author__ = 'Andrew Wen'

def ObjToJson(obj):
    """
    对象转换为字典，排除数据库属性
    :param obj:
    :return:
    """
    ctdict = obj.__dict__

    # 去掉数据库对象
    ctdict.pop('_sa_instance_state')
    ctdict["key"]=ctdict["id"]

    return ctdict


def JsonStrToObj(jsonStr,targetObj):
    """
    json转换到数据库对象
    :param obj:
    :return:
    """
    ctdict = json.loads(jsonStr)

    method = ctdict["method"]



    for key in ctdict:
        if hasattr(targetObj, key):
            setattr(targetObj, key, ctdict[key])



    return method,targetObj




def CopyObj(source,target):
    """
    拷贝对象属性，排除数据库相关特性
    :param source:
    :param target:
    :return: 负责完成属性的对象
    """
    sourcedict = source.__dict__

    # 去掉数据库对象
    sourcedict.pop('_sa_instance_state')

    for key in sourcedict:
        if hasattr(target, key):
            setattr(target, key, sourcedict[key])



    return target



def ObjToJsonStr(obj):
    ctdict = obj.__dict__

    # 去掉数据库对象
    ctdict.pop('_sa_instance_state')

    return json.dumps(ctdict,cls=CJsonEncoder)



def GetResult(pagination,result=None):
    """
    获得返回结果对象
    :param pagination:
    :return:
    """
    if result== None:
       result={"code":"0000","msg":"操作成功"}

    page= {
            'result':{"code":result["code"],"msg":result["msg"]},
            'list': [user.to_json() for user in pagination.items],
            'pagination':{'total': pagination.total, 'pageSize': pagination.per_page, 'current': pagination.page}
        }

    # {
    #     'list': [user.to_json() for user in pagination.items],
    #     'page': pagination.page,
    #     'prev_num': pagination.prev_num,
    #     'next_num': pagination.next_num,
    #     'has_next': pagination.has_next,  # 如果有下一页，返回True
    #     'has_prev': pagination.has_prev,  # 如果有上一页，返回True
    #     'pages': pagination.pages,  # 查询得到的总页数
    #     'per_page': pagination.per_page,  # 每页显示的记录数量
    #     'total': pagination.total,
    #     'iter_pages': [pages for pages in
    #                    pagination.iter_pages(left_edge=2, left_current=2, right_current=3, right_edge=2)]
    #
    # }

    return page;


from lib2to3.pytree import Base

from flask_login import UserMixin, AnonymousUserMixin
from sqlalchemy import Table, Column, Integer, ForeignKey
from werkzeug.security import generate_password_hash, check_password_hash
from itsdangerous import TimedJSONWebSignatureSerializer as Serializer
from datetime import datetime
from flask import current_app, request, url_for

from pyfw import db, login_manager

#用户角色多对多关系表
from pyfw.util import JsonUtil

user_role_mapper = db.Table('common_user_roles',
                         db.Column('user_id', db.Integer, db.ForeignKey('common_user_info.id') , nullable=False, primary_key=True),
                         db.Column('role_id', db.Integer, db.ForeignKey('common_role_info.id') , nullable=False, primary_key=True)
                         )


#用户组织多对多关系表
user_org_mapper = db.Table('common_user_org',
                         db.Column('user_id', db.Integer, db.ForeignKey('common_user_info.id') , nullable=False, primary_key=True),
                         db.Column('org_id', db.Integer, db.ForeignKey('common_org_info.id') , nullable=False, primary_key=True)
                         )





class CommonUserInfo(UserMixin,db.Model):
    """
    用户表
    """
    __tablename__ = 'common_user_info'

    id = db.Column(db.Integer, primary_key=True)
    # 账户
    login_account = db.Column(db.String(64), unique=True)
    # 密码
    login_password = db.Column(db.String(128))
    # 真实名称
    user_name = db.Column(db.String(64))
    # 用户编号
    user_no = db.Column(db.String(64))
    # 账号状态
    user_status = db.Column(db.String(64))
    # 所属系统
    user_sys = db.Column(db.String(64))
    # 手机号
    user_phone = db.Column(db.String(64))
    # 用户分类（1：内部人员；2：外部人员）
    user_type = db.Column(db.String(64))
    # 备注
    user_remark = db.Column(db.String(64))
    # 用户性别（1:男性，0:女性）
    user_sex = db.Column(db.String(64))
    # 性别
    user_gender = db.Column(db.Integer)
    # 邮箱
    user_email = db.Column(db.String(64))
    # 所属组织机构
    user_org = db.Column(db.Integer)

    # extend
    # 真实名称
    icon = db.Column(db.String(64))

    # 最后登录时间
    last_login = db.Column(db.DateTime(), default=datetime.utcnow)

    # 用户状态
    status = db.Column(db.Integer)
    # 操作人
    operate_user_id = db.Column(db.Integer)
    # 操作时间
    operate_time = db.Column(db.DateTime(), default=datetime.utcnow)

    roles = db.relationship('CommonRoleInfo',
                            secondary=user_role_mapper,
                            lazy='dynamic',
    backref = db.backref('users', lazy='dynamic')
    )

    orgs = db.relationship('CommonOrgInfo',
                            secondary=user_org_mapper,
                            lazy='dynamic',
                            backref=db.backref('users', lazy='dynamic')
                            )


    def __repr__(self):
        return '<common_user_info %r>' % self.__dict__

    @property
    def password(self):
        raise AttributeError('password is not a readable attribute')

    @password.setter
    def password(self, password):
        self.login_password = generate_password_hash(password)

    def verify_password(self, password):
        """
        密码验证方法
        :param password: 需要验证的密码
        :return:
        """
        return check_password_hash(self.login_password, password)

    @login_manager.user_loader
    def load_user(user_id):
        return CommonUserInfo.query.get(int(user_id))

    def to_json(self):

        return JsonUtil.ObjToJson(self)

    @staticmethod
    def generate_fake(count=100):
        from sqlalchemy.exc import IntegrityError

        from random import seed
        import forgery_py
        seed()
        for i in range(count):
            u = CommonUserInfo(user_email=forgery_py.internet.email_address(),
                               login_account=forgery_py.internet.user_name(True),
                               login_password=forgery_py.lorem_ipsum.word(),
                               icon="images/2.jpg"
                                )
            db.session.add(u)
            try:
                db.session.commit()
            except IntegrityError:
                db.session.rollback()






class CommonRoleInfo(db.Model):
    """
        角色表
    """
    __tablename__ = 'common_role_info'
    id = db.Column(db.Integer, primary_key=True)
    #角色代码
    role_code = db.Column(db.String(64), unique=True)
    #角色名称
    role_name = db.Column(db.String(64))
    #角色顺序
    role_order=db.Column(db.Integer)
    #备注
    role_remark=db.Column(db.Text)
    # 角色装
    role_status = db.Column(db.Integer)
    # 所属系统
    sys_id = db.Column(db.Integer)

    # 用户状态
    status = db.Column(db.Integer)
    # 操作人
    operate_user_id = db.Column(db.Integer)
    # 操作时间
    operate_time = db.Column(db.DateTime(), default=datetime.utcnow)



    def __repr__(self):
        return '<common_role_info %r>' % self.role_name





class CommonOrgInfo(db.Model):
    """
        组织机构表
    """
    __tablename__ = 'common_org_info'
    id = db.Column(db.Integer, primary_key=True)
    #组织机构代码
    org_code = db.Column(db.String(64), unique=True)
    # 组织机构名称
    org_name = db.Column(db.String(64), unique=True)
    #组织机构全称
    org_fullname = db.Column(db.String(64), unique=True)

    # 节点状态
    org_status = db.Column(db.Integer)

    org_area_id = db.Column(db.String(64))

    org_remark = db.Column(db.String(200))
    # 父节点编号
    org_parent_id = db.Column(db.Integer)
    # 排序
    org_sort  = db.Column(db.Integer)

    # 机构类别
    org_type  = db.Column(db.Integer)





    # 状态
    status = db.Column(db.Integer)
    # 操作人
    operate_user_id = db.Column(db.Integer)
    # 操作时间
    operate_time = db.Column(db.DateTime(), default=datetime.utcnow)





    def __repr__(self):
        return '<common_org_info %r>' % self.org_name





class CommonMenuInfo(db.Model):
    """
        菜单表
    """
    __tablename__ = 'common_menu_info'
    id = db.Column(db.Integer, primary_key=True)
    # 菜单代码
    menu_code = db.Column(db.String(64), unique=True)
    # 菜单名称
    menu_name = db.Column(db.String(64))
    # 菜单导航
    menu_nav = db.Column(db.String(64))
    # 备注
    menu_remark = db.Column(db.String(64))
    # url
    menu_url = db.Column(db.String(64))
    #菜单样式
    menu_cls = db.Column(db.String(64), unique=True)


    #菜单级别
    menu_level=db.Column(db.Integer)
    # 类别
    menu_type = db.Column(db.Integer)


    #菜单排序
    menu_order=db.Column(db.Integer)
    # 状态
    menu_status = db.Column(db.Integer)
    #父节点
    menu_pid=db.Column(db.String(64))
    #是否隐藏
    menu_hidden = db.Column(db.Integer)
    #布局
    menu_use_sys_layout = db.Column(db.Integer)
    #目标
    menu_target = db.Column(db.String(64))
    # 所属系统
    menu_sysid = db.Column(db.String(64))
    #spt
    menu_spt = db.Column(db.String(64))





    # 状态
    status = db.Column(db.Integer)
    # 操作人
    operate_user_id = db.Column(db.Integer)
    # 操作时间
    operate_time = db.Column(db.DateTime(), default=datetime.utcnow)



    def __repr__(self):
        return '<common_org_info %r>' % self.role_name



class CommonMenuOptInfo(db.Model):
    """
        菜单操作表
    """
    __tablename__ = 'common_menu_opt_info'
    id = db.Column(db.Integer, primary_key=True)
    #操作编号
    menu_id = db.Column(db.Integer)
    #操作代码
    opt_code = db.Column(db.String(64))
    #操作名称
    opt_name = db.Column(db.String(64))
    #操作url
    opt_url = db.Column(db.String(64))
    #操作方法
    opt_method = db.Column(db.String(64))
    #操作状态
    opt_status = db.Column(db.Integer)
    #操作备注
    opt_remark = db.Column(db.String(64))
    #排序
    opt_order = db.Column(db.Integer)

    # 状态
    status = db.Column(db.Integer)
    # 操作人
    operate_user_id = db.Column(db.Integer)
    # 操作时间
    operate_time = db.Column(db.DateTime(), default=datetime.utcnow)

    def __repr__(self):
        return '<common_org_info %r>' % self.opt_name